try:
    from rest_framework.validators import UniqueValidator, record_already_exists
    from rest_framework.exceptions import ValidationError
except ImportError:
    raise SystemExit(
        "Unable to import `record_already_exists`. "
        "Ensure that the Compass fork of Django Rest Framework is installed!"
    )


# ---------------------------------------------------------------------------
class LCUnique(UniqueValidator):

    def __init__(self, queryset, value, field='cid', message=None, lookup='exact'):
        super().__init__(queryset=queryset, message=message, lookup=lookup)

        self.field_name = field

    def __call__(self, value):
        queryset = self.filter_queryset(value, self.queryset, self.field_name)
        if existing_record_cid := queryset.values_list('cid', flat=True).first():
            raise ValidationError(
                self.message,
                code='unique',
                existing_record_cid=existing_record_cid,
            )


# ---------------------------------------------------------------------------
class PreCreateValidation:
    """
    Any validation that should only take place on 'create'.
    such as checking for unique values, because we so often 
    override the field-level UniqueValidator on nested serializers
    """

    def pre_create_validation(self, data):
        for field in ('cid', 'pk', 'id'):
            if field not in data:
                continue
            LCUnique(queryset=self.Meta.model.objects.all(), value=data[field], field=field)
        return data


__all__ = (
    'LCUnique',
    'PreCreateValidation',
)
