from django.db.models import IntegerChoices, TextChoices


def get_label(enum, value):
    """
    Get label from value...
    Python 3.9 enums don't like mixins...
    """
    try:
        return enum.__dict__['_value2member_map_'][value].label
    except KeyError:
        pass


def _get_value(enum, value):
    """
    Get Enum from raw value...
    Python 3.9 enums don't like mixins...
    """
    try:
        return enum.__dict__['_value2member_map_'][value]
    except KeyError:
        pass


class LCIntegerChoices(IntegerChoices):

    @classmethod
    def get_value(cls, value: int):
        return _get_value(cls, value)

    @classmethod
    def label_from_value(cls, value: int):
        return get_label(cls, value)


class LCTextChoices(TextChoices):

    @classmethod
    def get_value(cls, value: str):
        return _get_value(cls, value)

    @classmethod
    def label_from_value(cls, value: str):
        return get_label(cls, value)


# ---------------------------------------------------------------------------
class CSS(LCTextChoices):
    """
    Bootstrap CSS Class base names
    """
    Danger = 'danger', 'danger'
    Dark = 'dark', 'dark'
    Default = 'default', 'default'
    Info = 'info', 'info'
    Light = 'light', 'light'
    Primary = 'primary', 'primary'
    Secondary = 'secondary', 'secondary'
    Success = 'success', 'success'
    Transparent = 'transparent', 'transparent'
    Warning = 'warning', 'warning'
    White = 'white', 'white'
