from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
from lchttp import json_dumps
from unpoly.typehints import UnpolyHttpRequest
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from sandbox.latchstring.models import User


class LCUnpolyHttpRequest(UnpolyHttpRequest):
    user: 'User'
    impersonator: 'User'
    real_user: 'User'


class FastJsonResponse(HttpResponse):
    """
    Implementation of Django's JsonResponse, using the ``json_dumps``
    library for performance.
    """

    def __init__(
        self,
        data,
        encoder=DjangoJSONEncoder,  # noqa
        safe=True,
        json_dumps_params=None,  # noqa
        **kwargs,
    ):
        if safe and not isinstance(data, dict):
            raise TypeError(
                'In order to allow non-dict objects to be serialized set the '
                'safe parameter to False.'
            )
        kwargs.setdefault('content_type', 'application/json')

        data = json_dumps(data)

        super().__init__(content=data, **kwargs)


__all__ = (
    'LCUnpolyHttpRequest',
    'FastJsonResponse',
)
