from django.contrib.postgres.indexes import (
    GinIndex,
    GistIndex,
    BTreeIndex,  # noqa  convenience import
)


class MultiColumnSql:
    """
    Index normal column and UPPER(column_name) so the index can be used
    for both <fieldname>__trigram_similar and <fieldname>__icontains queries.
    """

    def create_sql(self, model, schema_editor, using='', **kwargs):
        """
        For each column in the index add the same column with UPPER(column)
        """
        statement = super().create_sql(model, schema_editor, using=using, **kwargs)  # noqa

        # get columns, which should be comma-separated column name & ops class, per index type.
        # "name" gist_trgm_ops , "code" gist_trgm_ops
        # "name" gin_trgm_ops , "code" gin_trgm_ops
        standard_columns = str(statement.parts["columns"])

        upper_columns = []
        for column in standard_columns.split(','):
            name, op_class = column.strip().split()
            upper_columns.append(f'UPPER({name}) {op_class}')

        statement.parts["columns_upper"] = f'{standard_columns} , {" , ".join(upper_columns)}'
        statement.template = statement.template.replace('columns', 'columns_upper')

        return statement


class UpperGistIndex(MultiColumnSql, GistIndex):
    """
    Index normal column and UPPER(column_name) so the index can be used
    for both <fieldname>__trigram_similar and <fieldname>__icontains queries.
    """

    def __init__(self, *, buffering=None, fillfactor=None, **kwargs):  # noqa
        super().__init__(**kwargs)


class UpperGinIndex(MultiColumnSql, GinIndex):
    pass
