from dataclasses import dataclass
from datetime import datetime
from typing import NamedTuple, Optional, Sequence, Union
from uuid import UUID

PrimaryKey = Union[int, UUID]
OptionalPrimaryKey = Optional[PrimaryKey]
CanonicalID = UUID
CanonicalIDString = str
CanonicalIDs = list[CanonicalIDString]
FIELDS = tuple[str, str]


@dataclass(slots=True, frozen=True)
class View:
    name: str
    icon: str
    url: str
    url_name: str


@dataclass(slots=True, frozen=True)
class Partition:
    name: str
    start_date: datetime | None
    end_date: datetime | None


class LookupField(NamedTuple):
    name: str
    required: bool


FILTER_FIELDS = Sequence[Union[FIELDS, tuple[FIELDS, str]]]
